#!/usr/bin/perl -w
# %Z%%M%------- %I%- %W% %G% %U%
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2009, 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.

use warnings;
use strict;
use Getopt::Long;

my %options = ();
my $IFCFGFILE = "/etc/sysconfig/network/ifcfg-";
my $ROUTEFILE = "/etc/sysconfig/network/routes";
my $HOSTNAMEFILE = "/etc/HOSTNAME";
my $HOSTFILE = "/etc/hosts";
my $DHCPCONFFILE = "/etc/dhcpd.conf";
my $DHCPDFILE = "/etc/sysconfig/dhcpd";
my $LEASEFILE = "/var/lib/dhcp/db/dhcpd.leases";

################################################################################
#  Get all possible parameters
################################################################################

if (! (GetOptions (\%options, 'enabled=s', 'interface=s','networktype=s','ipversion=s', 'iptype=s','dhcptimeout=i','ipv4address=s','netmask=s','ipv6address=s','startipaddr=s','endipaddr=s', 'subnet=s','router=s'))) {
    exit (1);
}


################################################################################
#  Check parameters
################################################################################

sub checkOption
{

    my ($option, @validValue) = @_;

    if (!defined $options{$option}){
        print "AE: Property \"$option\" not specified. Ignore the setting.\n";
        exit 1;
    }
    my $attr = $options{$option};

    # if the option has no value list, return the value directly
    if (@validValue == 0) {
        return $attr;
    }
    # check the format of the value if option is 'dhcptimeout'
    if ($option eq "dhcptimeout") {
        if ($attr =~ /^[0-9]{1,3}$/) {
            return $attr;
        }
    }
    else {
        for my $v (@validValue){
            if ($attr eq $v) {
                return $v;
            }
        }
    }
    print "AE: Property \"$option\" value can only be @validValue.\n";
    exit 1;
}

################################################################################
# check if this interface is enabled, if "no", ignore the rest configuration
################################################################################

my $enabled=checkOption 'enabled', ('yes', 'no');
if ($enabled eq 'no'){
    print "AE: interface setting not enabled. Ignore the setting.\n";
    exit 0;
}

################################################################################
# get the network interface name
################################################################################

my $interface=checkOption 'interface', ('eth0', 'eth1', 'eth2', 'eth3');
$IFCFGFILE .= $interface;  # the ifcfg-eth* file


################################################################################
# The network can by confiugred as a standard network interface or a DHCP server
################################################################################

my $networktype=checkOption 'networktype', ('standard','dhcpserver');

# if the adapter is configured as a standard network interface
if ($networktype eq 'standard') {
# Configure one or both of IPv4 address and IPv6 addresses

     my $iptype=checkOption 'iptype', ('static','dynamic', 'dhcp-then-static');
     if ($iptype eq 'dynamic') {
        # add "supersede router 0.0.0.0;" to /etc/dhclient.conf for avioding get default dateway from dhcp server
        my $IFDHCPFILE="/etc/dhclient.conf";
        my $super_router=0;
        my @DHCLIENT=`cat /etc/dhclient.conf`;
        for(@DHCLIENT) {
           chomp;
           if(/^supersede router 0.0.0.0;$/){
              $super_router=1;
              last;
            }
        }
        if($super_router == 0){
           open DHCPCFG, ">>$IFDHCPFILE" or die "cannot open dhcp configuration file to write.\n";
           print DHCPCFG "supersede router 0.0.0.0;\n";
           close DHCPCFG;
        }

        # Configure the interface as a DHCP client
        my $ip_conf="BOOTPROTO='dhcp'\nIPADDR=\nMTU='1500'\nSTARTMODE='onboot'\nNETMASK=\n";
        open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
        print IFCFG $ip_conf;
        close IFCFG;
     }elsif ($iptype eq 'static') {
           # Configure static Ip address

           my $ipv4_conf="";
           my $ipv6_conf="";
           my $ipversion=checkOption 'ipversion', ('ipv4','ipv6','mix');
           if ($ipversion eq 'ipv4' || $ipversion eq 'mix') {
           # Configure static IPv4 address
                my $ipv4address=checkOption 'ipv4address';
                my $netmask=checkOption 'netmask';
                $ipv4_conf .= "BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n"."IPADDR='$ipv4address'\nNETMASK='$netmask'\n";
                open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
                print IFCFG $ipv4_conf;
                close IFCFG;
           }
           if ($ipversion eq 'ipv6' || $ipversion eq 'mix') {
           # Configure static IPv6 address
                my $ipv6address=checkOption 'ipv6address';
                if ($ipv4_conf eq ""){
                   # This line could be duplicate if static IPv4 address has been configured
                   $ipv6_conf.="BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n";
                }else{
                   $ipv6_conf.=$ipv4_conf;
                }
                my $i=0;
                foreach my $addrv6 (split /,/,$ipv6address){
                   $i++;
                   $ipv6_conf.="IPADDR$i='$addrv6'\n";
                }
                open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
                print IFCFG $ipv6_conf;
                close IFCFG;
           }
     }else {
           my $dhcptimeout=checkOption 'dhcptimeout', ('<0-999>');

           # add "supersede router 0.0.0.0;" to /etc/dhclient.conf for avioding get default dateway from dhcp server
           my $IFDHCPFILE="/etc/dhclient.conf";
           my $super_router=0;
           my @DHCLIENT=`cat /etc/dhclient.conf`;
           for(@DHCLIENT) {
              chomp;
              if(/^supersede router 0.0.0.0;$/){
                $super_router=1;
                last;
              }
            }
            if($super_router == 0){
              open DHCPCFG, ">>$IFDHCPFILE" or die "cannot open dhcp configuration file to write.\n";
              print DHCPCFG "supersede router 0.0.0.0;\n";
              close DHCPCFG;
            }

           # Configure the interface as a DHCP client
           my $ip_conf="BOOTPROTO='dhcp'\nIPADDR=\nMTU='1500'\nSTARTMODE='onboot'\nNETMASK=\n";
           open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
           print IFCFG $ip_conf;
           close IFCFG;

           # Restart network
           #`service network restart`;

           # Test DHCP client
           my $dhcp_result = 0;

           my @dhclient_result = `dhclient -n $interface 2>&1`;
           for (@dhclient_result) {
               chomp;
               if (/No DHCPOFFERS received/) {
                   $dhcp_result = 0;
                   last;
               }
               if (/DHCPACK from/) {
                   $dhcp_result = 1;
                   last;
               }
           }
           sleep($dhcptimeout);



=tuc
           # Wait for a specified period of time and check DHCP configuration
           sleep($dhcptimeout);
           my $ipv4addr = `lsnetcfg -n | grep "$interface" | grep "ipv4addr_$interface"`;
           my $ipv4dhcp = `lsnetcfg -n | grep "$interface" | grep "ipv4dhcp_$interface"`;
           chomp($ipv4addr);
           chomp($ipv4dhcp);
           if ($ipv4dhcp =~ /ipv4dhcp_$interface: off/) { $dhcp_result = 0; }
           elsif ($ipv4dhcp =~ /ipv4dhcp_$interface: on/ && $ipv4addr =~ /ipv4addr_$interface: (25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9])\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[1-9]|0)\.(25[0-5]|2[0-4][0-9]|[0-1]{1}[0-9]{2}|[1-9]{1}[0-9]{1}|[0-9])/) { $dhcp_result = 1; }

=cut

           # If DHCP configuration failed, configure static IP address
           if ($dhcp_result == 0) {

           # Configure static Ip address
           my $ipv4_conf="";
           my $ipv6_conf="";
           my $ipversion=checkOption 'ipversion', ('ipv4','ipv6','mix');
           if ($ipversion eq 'ipv4' || $ipversion eq 'mix') {
           # Configure static IPv4 address
                my $ipv4address=checkOption 'ipv4address';
                my $netmask=checkOption 'netmask';
                $ipv4_conf .= "BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n"."IPADDR='$ipv4address'\nNETMASK='$netmask'\n";
                open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
                print IFCFG $ipv4_conf;
                close IFCFG;
           }
           if ($ipversion eq 'ipv6' || $ipversion eq 'mix') {
           # Configure static IPv6 address
                my $ipv6address=checkOption 'ipv6address';
                if ($ipv4_conf eq ""){
                   # This line could be duplicate if static IPv4 address has been configured
                   $ipv6_conf.="BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n";
                }else{
                   $ipv6_conf.=$ipv4_conf;
                }
                my $i=0;
                foreach my $addrv6 (split /,/,$ipv6address){
                   $i++;
                   $ipv6_conf.="IPADDR$i='$addrv6'\n";
                }
                open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
                print IFCFG $ipv6_conf;
                close IFCFG;
           }

           `service network restart`;

           }

    }

}
else{
     ############################
     #  configure DHCP server
     ###############################



     # Config /etc/dhcpd.conf file
     my $ipv4address=checkOption 'ipv4address';
     my $startipaddr=checkOption 'startipaddr';
     my $endipaddr=checkOption 'endipaddr';
     my $netmask=checkOption 'netmask';
     my $subnet=checkOption 'subnet';
     my $router=checkOption 'router';
     my $dhcp_server_conf="";
     $dhcp_server_conf.="# Licensed Internal Code - Property of IBM\n";
     $dhcp_server_conf.="#    A sample dhcpd.conf file\n";
     $dhcp_server_conf.="#  (Copy this to /etc/dhcpd.conf\n";
     $dhcp_server_conf.="#\n";
     $dhcp_server_conf.="# authoritative;\n";
     $dhcp_server_conf.="ddns-update-style none;\n";
     $dhcp_server_conf.="default-lease-time 600;\n";
     $dhcp_server_conf.="max-lease-time infinite;\n";
     $dhcp_server_conf.="subnet $subnet netmask $netmask {\n";
     $dhcp_server_conf.="option routers $router;\n";
     $dhcp_server_conf.="  range $startipaddr  $endipaddr;\n";
     $dhcp_server_conf.="}\n";
     open DHCPCFG, ">$DHCPCONFFILE" or die "AE: cannot open interface configuration file to write.\n";
     print DHCPCFG $dhcp_server_conf;
     close DHCPCFG;


     # Config /etc/syscofig/dhcpd
     `cat /etc/sysconfig/dhcpd | sed "s/^DHCPD_INTERFACE=\".*\"/DHCPD_INTERFACE=\"${interface}\"/g"  > /etc/sysconfig/dhcpd.tmp`;
     `cp /etc/sysconfig/dhcpd.tmp /etc/sysconfig/dhcpd`;
     `rm /etc/sysconfig/dhcpd.tmp`;
     # Make sure the leases file is there before we try to start
     unless (-e $LEASEFILE){
             `touch ${LEASEFILE}`;
     }
     my $ipv4_conf = "BOOTPROTO='static'\nMTU='1500'\nSTARTMODE='onboot'\n"."IPADDR='$ipv4address'\nNETMASK='$netmask'\n";
     open IFCFG, ">$IFCFGFILE" or die "AE: cannot open interface configuration file to write.\n";
     print IFCFG $ipv4_conf;
     close IFCFG;
}

exit 0;
