#!/bin/bash
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2010, 2011 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

PATH=/usr/bin:$PATH
PROGNAME=$(basename $0)
PROGPATH=$(dirname $0)

INSTALL_DIR=/opt/ibm/director
CMD_DIR=$INSTALL_DIR/ha/bin/priv
COMMON_CODE=/opt/ibm/director/ha/bin/include/smhacommon.sh

if [ -f $COMMON_CODE ]; then
    source $COMMON_CODE
else
    echo "$PROGNAME:Common code not found."
    exit 1
fi
#############################################################
load_common_code $INSTALL_DIR/ha/bin/include/smhainitcom.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadir.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhaagent.sh
load_common_code $INSTALL_DIR/ha/bin/include/smharsrc.sh
load_common_code $INSTALL_DIR/ha/bin/include/smhadrbd.sh

wait_for_sam () {

    while true; do
		
	    local ctrl_out=`LC_ALL=C lssamctrl 2>&1`
	    ctrl_rc=$?
	
	    if [ $ctrl_rc -eq 0 ]; then
	        debug "HA: $ctrl_out"
	        local automation=`echo $ctrl_out | grep Automation`
	        if [ -n "$automation" ]; then        
	            return
	        else
	            debug "Waiting for sam: rc=$ctrl_rc"
	            sleep 10
	        fi
	    else
	        debug "Waiting for sam: rc=$ctrl_rc"
	        sleep 10
	    fi

    done
}

wait_for_director_rg () {
	
	local pending=0
    for i in {1..96}
    do
        get_rg_status_num $DIRECTOR_RG
        # 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=pending offline        
        case $? in
        1|4 )
            return 0
        ;;        
        5 )
            pending=1
            sleep 5
        ;;
        3 )
            # If it's failed offline and we previously saw it starting, then return
            if [ $pending -eq 1 ]; then
            	return 0
            else
                sleep 5
            fi
        ;;
        *)
        # Otherwise, just wait until possibly our time out
            sleep 5
        ;;
        esac
    done
	
}


hdl_cluster_version_updates () {
	
	run_init clustervrm
	RC=$?
	return $RC
}

###################################
## Main code                     ##
###################################
RC=0
debug "smhamaint called"

#if the stop maint flag is on, then don't run smhamaint
if [ -e $STOP_MAINT_FLAGFILE ]; then
    delete_stop_maint_flagfile
    debug "stop maint flag is on...so exiting smhamaint"
    exit 0
fi

mutex_clear_all

rm -f ${HACMD_INPROGRESS}*    # clear "command in progress" flag files, if they exist

# Do not use is_configured because at this point (when the appliance
# is being started up), is_configured can sometimes return false
# even when HA really is configured.  Instead call is_HAconfigured, which
# does not call RSCT, which may still be coming up.

is_HAconfigured
isHAConfigured=$?

if [ $isHAConfigured -eq 0 ]; then
	debug "HA is not configured."
else

    # Reset the dirc settings to passive, they'll get set to active if director is started.
    dirc_use_settings passive

    # check if ctrmc start has been "disabled". If so, this is the first boot after a restore
    # We need to call ctrestore and re-enable ctrmc start.
    #
    # N.B. the ctrestore will start the TSA/RSCT stack

    if [ -f /etc/init.d/ctrmc.HAsaved ]
    then
        # set the necessary flag files so the DRBD logic knows to re-create the mirror 
        # metadata and to force the local mirrors to come up as primary.
        #
        # See the comments for the set_port_restore_mirror_flags function in include/smhadrbd.sh

        set_post_restore_mirror_flags

        rm -f /etc/init.d/ctrmc
        mv /etc/init.d/ctrmc.HAsaved /etc/init.d/ctrmc

        if /usr/sbin/rsct/bin/ctrestore &> $BKRT_CTRESTORE_OUT
        then
            debug "ctrestore succeeded"
            rm -f $BKRT_CTRESTORE_OUT
            rm -rf /var/ct.backup
        else
            # the TSA/RSCT stack is in an unknown state; cannot go on
            fatal_error "ctrestore failed; see $BKRT_CTRESTORE_OUT"
        fi
    fi
    
    # call cleanup handler plug-in for setting time backward (via chconfig). If the node rebooted between
    # the "pre" and "post" steps, this  will get TSA restarted.

    /opt/ibm/director/ha/bin/priv/smhachktime.sh chconfig.d cleanup

    wait_for_sam

    # check for presence of backup/restore flag file. If it is present, this is the first
    # boot after a restore. The file contains the NominalState values saved just prior to
    # the backup. In the TSA/RSCT backup data, the NominalState values are all Offline.

    if [ -f $BKRT_FLAGFILE ]
    then
        if ! restore_rgs_state $BKRT_FLAGFILE
        then
            debug "Could not restore NominalState values"
        fi
        rm -f $BKRT_FLAGFILE
    fi
    
    # check on every IPL - if RSCT or TSA version has moved forward, then process appropriately
    hdl_cluster_version_updates
    
    # if automation flag file is found, then handle it.
    if [ -e $AUTOMATION_FLAGFILE ]; then
    	is_automation_on
    	if [ $? -eq 0 ]; then
    		debug "Automation is not on, going into single node mode"
	    	set_single_node_mode 
	    	turn_on_automation
	    	delete_automation_flagfile
	    	# Wait enough time to get the resources starting on the desired node
	    	sleep 120
	    	wait_for_director_rg
	    	set_dual_node_mode
    	else
            # Automation is already on, so just delete the flag and ignore it.
            debug "Automation already turned on."
            delete_automation_flagfile    	   
        fi
    fi
        
    # if the shutdown halt flag is on, then handle it.
	handle_shutdown_halt_flagfile

    # if the SyncConf is needed.  This must be done after we include this node
    # in the domain since we post a msg to our mq.
    #if [ -e $SYNCCONFIG_FLAGFILE ]; then
        # This is needed when the ip address changes and the system is rebooted
        # Send the sync request to the other node.
           # sync_tsa_config
           
    #    debug "SyncConfig"
    #    run_init "SyncConfig" 
        
    #    delete_syncconf_flagfile
    #fi
    
    # if the refreshManagementIP flag is on
    if [ -e $REFRESH_MANAGEMENTIP_FLAGFILE ]; then
    	# wait for director to come online, 
    	# and then refresh management IPs as needed.
    	wait_for_director_rg
    	refresh_managementIPs
    	# delete the file now
    	delete_refresh_managementIP_flagfile
    fi 
    
    # if the exclude peer node flag is on, then handle it.
 	handle_exclude_peer_flagfile

    # Always restore the resource protection.  If we're the only node up then it'll fail and
    # the protection will still be off (which is what we want.)
    set_crit_rsrc_protection $RSCT_CRIT_RSRC_PROTECT__HARD_RESET
    
fi


debug "smhamaint exit; RC=$RC"
exit $RC
