# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2011, 2012 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END

#
# 09N6996
#

# N.B. all functions prefixed by _smha are private and not to be used
# by exploiters

_smha_log () {
    
    # Only log if the log directory exists
    if [ -d "${_SMHA_LOG_DIR}" ]; then
 
        local trc_data=
        if [ -n "${_SMHA_LOG_ID}" ]; then
           trc_data="${_SMHA_LOG_ID}:${PROGNAME}"
        else
           trc_data="$PROGNAME"
        fi
        echo -e `date +%F\ %T.%N` "$trc_data: $*" >> ${_SMHA_LOG}
        
    fi  
}

_smha_gen_log_id () {
    if [ -z "${_SMHA_LOG_ID}" ]; then
        local nano=`date +%N`
        _SMHA_LOG_ID=${nano:3:4}
    fi
}

_smha_debug_internal () {

    # Only print to the screen if debug is on
    if [[ "$SMHA_DEBUG" == "ON" ]]; then
        echo "$*"
    fi
    # Always place debug in the log
    _smha_log "$*"
}

# The following function is used to log messages to the
# Director HA log file /opt/ibm/director/ha/logs/smha.log
# The function arguments are logged as a single message
# prefixed by a timestamp, a unique ID and the program name.
# This function should be used to log information useful for
# debugging by support, i.e. FFDC type information.
#
# if SMHA_DEBUG=ON, then the message is also written to stdout

smha_debug () {

    # Check if we only want to debug one program
    if [[ -n "$SMHA_DEBUG_PGM_ONLY" ]]; then
        # Only debug if this is the program to debug
        if [[ "$PROGNAME" == "$SMHA_DEBUG_PGM_ONLY" ]]; then
            _smha_debug_internal "$*"
        fi
    else
        _smha_debug_internal "$*"
    fi
}

# The following function is similar to smha_debug, but only
# logs the message if SMHA_LOG_VERBOSE=verbose. This function
# should be used to capture messages useful during development of
# the exploiter script but should be turned off for the shipped
# code.

smha_debug_verbose () {
    if [[ "$SMHA_LOG_VERBOSE" == "verbose" ]]; then
        smha_debug "$*"
    fi
}

# The following function is similar to smha_debug, but only
# logs the message if SMHA_DEBUG_TSA=ON. It will also prefix the
# passed arguments with the string "TSA: "

smha_debug_tsa () {
    # TSA scripts cause a lot of debug to the log. Only debug if if DEBUG_TSA is on.
    if [[ "$SMHA_DEBUG_TSA" == "ON" ]]; then
        smha_debug "TSA: $*"
    fi
      
}

# The following function should be used to create an "audit trail"
# of the actions performed by the scripts specified to 
# IBM.Application resources. The logged data is prefixed by the
# string "TSA_ACTION:". This string is used by the smhastat.sh
# script to report interesting events during TSA automation.
#
# Arg1:  action name, e.g. "start", "stop", "status"
# Arg2:  resource name (the Name attribute value of the IBM.Application resource)
# Arg3:  a return code indicating the success or failure of the action

smha_record_tsa_action () {
    local action=$1
    local name=$2
    local rc=$3
    shift 3

    smha_debug "TSA_ACTION:$action:$name:$rc:"
}

# The following function is used to log the single argument to
# the syslog, i.e. /var/log/messages

smha_sys_log () {
    logger -i -t "$PROGNAME" ${1}
}

# If the exploiter script encounters an unrecoverable error, the
# following function can be called. It will log the arguments to
# both the syslog and the HA log, then the calling process exits
# with the indicated exit code.
#
# Arg1:    exit code
# Arg2-n:  remaining arguments are the message to be logged

smha_fatal_error () {
    # The first parameter must be the exit code
    local RC=$1
    shift

    smha_sys_log "$*"
    _smha_log "$*"

    exit $RC
}

# The following function is used to determine if the calling process is
# running on the "passive" node.
#
# Return value:  0   this is the active node
#                1   this is the passive node

smha_is_passive () {
    local OpState
    local RC
    local Sel="Name == \"drbdp-mirror0\" && ResourceType == 0"

    OpState=$(CT_MANAGEMENT_SCOPE=1 lsrsrc-api -s IBM.Application::"${Sel}"::OpState 2> /dev/null)
    RC=$?
    if [ $RC -ne 0 ]
    then
        smha_fatal_error 1 "Cannot list IBM.Application resource drbdp-mirror0"
    fi

    case $OpState in
        # 0=unknown 1=online 2=offline 3=failed_offline 4=stuck_online 5=pending_online 6=pending_offline
        1|5 ) return 0 ;;
        *   ) return 1 ;;
    esac

}

# The following function is used to determine if the resource group is marked as "enabled" for the
# controlled plugin start/stop framework
#
# Input parameters:
# Arg 1  - Resource group name
#
# Return value:  0   the resource group is "disabled"
#                1   the resource group is "enabled"

smha_is_rg_enabled () {
    local RGName=${1}
    local OwnerAttrVal
    local RC

    # check if this is a "switchable" resource group, i.e. the Owner attribute value is not NULL
    OwnerAttrVal=$(LC_ALL=C lsrsrc-api -s "IBM.ResourceGroup::Name == \"${RGName}\"::Owner" 2>&1)
    RC=$?
    if [ $RC -eq 0 ]; then
        if [ -n "$OwnerAttrVal" ]; then
            if [ "$OwnerAttrVal" = "Enabled" ]; then
               return 1
            fi
        fi
    fi
    return 0
}

# The following function writes the local RSCT node name to stdout.
# This command should be invoked via command substitution, e.g.
#   loc_node_name=$(smha_get_local_rsct_node_name)

smha_get_local_rsct_node_name () {
    local node_name
    local RC
 
    # N.B. a management scope of 1 indicates "local" scope; the RMC subsystem implicitly
    # selects only locally defined fixed resources. The IBM.PeerNode class only defines one
    # resource per node in an online peer domain.

    node_name=$(CT_MANAGEMENT_SCOPE=1 lsrsrc-api -s IBM.PeerNode::::Name 2> /dev/null)
    RC=$?
    if [ $RC -ne 0 ]; then
        echo "fatal error for local"
        smha_fatal_error 1 "Cannot list local IBM.PeerNode resource"
    fi
   
    echo $node_name
}

# The following function write the remote RSCT node name to stdout.
# This command should be invoked via command substitution, e.g.
#   rem_node_name=$(smha_get_remote_rsct_node_name)

smha_get_remote_rsct_node_name () {
    local node_name
    local RC
    local NodeID
        
    # N.B. given HA is currently supporting only a two node peer domain, selecting a
    # resource that does not match the local node ID must provide the remote node's resource

    NodeID=$(/usr/sbin/rsct/bin/lsnodeid)

    node_name=$(lsrsrc-api -s IBM.PeerNode::"NodeIDs != {0x$NodeID}"::Name 2> /dev/null)
    RC=$?
    if [ $RC -ne 0 ]; then
        echo "fatal error for remote"
        smha_fatal_error 1 "Cannot list remote IBM.PeerNode resource"
    fi
   
    echo $node_name
}

#
# Wrapper for the TSA addrgmbr command. This function will retry the TSA command
# on error, every 5 seconds for a total of 100 attempts.
#
# When adding a resource to a resource group, all resources in the group must
# be offline. But a resource may be created and added to the group, where the
# initial state of the resource is online. Therefore, it is possible that the
# addition of another member to the group may be rejected.
#
# Input parameters 
# Arg 1  - Resource name
# Arg 2  - Resource class name
# Arg 3  - Group name
# Arg 4  - Select Policy (order), one of the following
#              A   (Any)
#              O   (Ordered) - Default option if not specified
#              O,F (Ordered with Failback)
# Arg 5  - Mandatory, one of the following
#              T   (True) - Default option if not specified
#              F   (False)
#
# Exit code: 0 - resource added to group
#            1 - resource could not be added to group
#
# N.B. This function can add only one member per call

smha_add_to_rg () {
    local name=${1}
    local type=${2}
    local group=${3}
    local order=${4}
    local mandatory=${5}
    local cmd_out

    if [ -n "$order" ]; then
        local order_flag="-p $order"
    fi

    if [ -n "$mandatory" ]; then
        local mandatory_flag="-m $mandatory"
    fi

    local i
    for i in {1..100}
    do

        cmd_out=`LC_ALL=C addrgmbr $order_flag $mandatory_flag -g $group $type:$name 2>&1`
        if [ $? -eq 0 ]; then
            smha_debug "addrgmbr for $name worked."

            return 0
        else
            smha_debug "addrgmbr output:$cmd_out"
            smha_debug "Retry adding $name to $group, order:$order_flag"
            sleep 5
        fi
    done
    return 1
}

#
# Wrapper for the TSA mkrel command. This function will retry the TSA command
# on error, every 5 seconds for a total of 60 attempts.
#
# When creating a relationship between resources, the related resources must
# be offline. But if the source or target resources are recently created,
# the initial state of the resource may be online. Therefore, it is possible
# that the new relationship is rejected.
#
# Input parameters 
# Arg 1  - relationship (argument to the mkrel -p option)
# Arg 2  - source (argument to the mkrel -S option)
# Arg 3  - target (argument to the mkrel -G option)
# Arg 4  - name of relationship
#
# Exit code: 0 - relationship made
#            1 - relationship could not be made

smha_make_relationship () {
    local relation=${1}
    local source=${2}
    local target=${3}
    local name=${4}
    local rname
    local cmd_out

    local i
    for i in {1..60}
    do

        smha_debug "mkrel -p $relation -S $source -G ${target} ${name}"
        cmd_out=$(LC_ALL=C mkrel -p $relation -S ${source} -G ${target} ${name} 2>&1)

        rname=$(lsrsrc-api -s "IBM.ManagedRelationship::Name == \"${name}\"::Name" 2> /dev/null)
        if [[ ${name} = ${rname} ]]; then
            smha_debug "Made relationship $name"

            return 0
        else
            smha_debug "mkrel cmd_out: $cmd_out"
            smha_debug "Retry making relationship $name"
            sleep 5
        fi
    done
    return 1
}

#
# Wrapper for the TSA rmrgmbr command. This function will retry the TSA command
# on error, every 5 seconds for a total of 60 attempts.
#
# This method will remove a resource from a resource group
#
# Input parameters 
# Arg 1  - Resource name
# Arg 2  - Group name
# Arg 3  - Resource type
#
# Exit code: 0 - resource removed from group
#            1 - resource could not be removed from group

smha_remove_resource () {
    local name=${1}
    local group=${2}
    local type=${3}
    local cmd_out
    
    local i
    for i in {1..60}
    do
        smha_debug "rmrgmbr -g $group $type:$name"
        cmd_out=`LC_ALL=C rmrgmbr -g $group $type:$name 2>&1`
    
        if [ $? -eq 0 ]; then
            smha_debug "rmrgmbr for $name worked."

            return 0
        else
            smha_debug "rmrgmbr output:$cmd_out"
            smha_debug "Retry removing $name from $group"
            sleep 5
        fi
    done
    return 1
}

#
# Wrapper for the TSA rmrel command. This function will retry the TSA command
# on error, every 5 seconds for a total of 60 attempts.
#
# Input parameters 
# Arg 1  - relationship (argument to the rmrel -S option)
#
# Exit code: 0 - relationship removed
#            1 - relationship could not be removed

smha_remove_relationship () {
    local name=${1}
    local cmd_out

    local i
    for i in {1..60}
    do
        smha_debug "rmrel -S $name"
        cmd_out=`LC_ALL=C rmrel -S $name 2>&1`
        
        if [ $? -eq 0 ]; then
            smha_debug "rmrel for $name worked."

            return 0
        else
            smha_debug "rmrel output:$cmd_out"
            smha_debug "Retry removing $name"
            sleep 5
        fi
    done
    return 1
}

#
# Wrapper for the TSA rmrg command. This function will retry the TSA command
# on error, every 5 seconds for a total of 60 attempts.
#
# Note that the rmrg command removes the group and all its managed resource
# members. But the target of each member, i.e. an IBM.Application resource, must
# be removed explicitly. This function removes each target resource.
#
# Input parameters 
# Arg 1  - Group name
#
# Exit code: 0 - resource group removed
#            1 - resource group could not be removed

smha_remove_resource_group () {
    local group=${1}
    local rg_status
    local RC
    local NameList
    local Name
    local RSRCname
    local List
    local RH
    local cmd_out
    
    # 0=unknown,1=online,2=offline,3=failed offline,4=stuck online,5=pending online,6=pending offline
    rg_status=$(lsrsrc-api -s "IBM.ResourceGroup::Name == '${group}'::OpState" 2> /dev/null)
    RC=$?

    if [ $RC -eq 6 ]; then
        smha_debug "Resource group $group does not exist"
        return 0
    elif [ $RC -ne 0 ]; then
        smha_debug "Cannot obtain status for resource group $group"
        return 1
    fi
    
    #if the resource group isn't offline, take it offline since the resource
    # group must be offline before it can be removed
    if [ $rg_status -ne 2 ]; then

        local i
        for i in {1..60}
        do
            smha_debug "chrg -o offline $group"
            cmd_out=`LC_ALL=C chrg -o offline $group 2>&1`
        
            if [ $? -eq 0 ]; then
                smha_debug "chrg for $group worked."
                break
            else
                smha_debug "chrg output:$cmd_out"
                smha_debug "Retry stopping $group"
                sleep 5
            fi
        done
    fi

    # get the resource handle for each of the managed resources in this group and,
    # for each resource handle, get the target resource name

    NameList=$(lsrsrc-api -s "IBM.ManagedResource::MemberOf == '$group'::ManagedResource" 2> /dev/null | \
    while read RH
    do
        RSRCname=$(lsrsrc-api -r "$RH::Name" 2> /dev/null)
        echo "$RSRCname"
    done)

    # remove the resource group and member resources

    RC=1
    for i in {1..60}
    do
        smha_debug "rmrg $group"
        cmd_out=`LC_ALL=C rmrg $group 2>&1`
    
        if [ $? -eq 0 ]; then
            smha_debug "rmrg for $group worked."

            RC=0
            break
        else
            smha_debug "rmrg output:$cmd_out"
            smha_debug "Retry removing $group"
            sleep 5
        fi
    done

    if [ $RC -eq 0 ]
    then
        # remove the target resources
        if [ -n "NameList" ]
        then
            List=
            for Name in $NameList
            do
                if [ -z "$List" ]
                then
                    List="'$Name'"
                else
                    List="${List},'$Name'"
                fi
            done

            smha_debug "Removing resources named $List"
            rmrsrc-api -s "IBM.Application::Name |< {$List}::Force::1"
        fi
    fi

    return $RC
}

# The following function is invoked by an Activation script when it has just created
# its resources and resource group. This function makes the resource group online if
# director is active; otherwise it will do nothing. In the latter case the activation
# script is being executed during HA configuration and the resource group cannot be
# made online at that point in time. HA configuration will make the resource group
# online when appropriate
#
# Input parameters 
# Arg 1  - Group name
#

smha_activate_rg_online () {
    local group=${1}
    local RC

    /opt/ibm/director/ha/bin/tsa/smhadirstatus.sh &> /dev/null
    RC=$?
    smha_debug "Activation: director status is $RC"

    if [ $RC -eq 0 -o $RC -eq 1 ]    # if active or starting
    then
        smha_debug "Activation: make $group online"
        chrg -o online $group
    else
        smha_debug "Activation: do not make $group online"
    fi

    return 0
}

# The following function is invoked by a Controlled script to create its resource group in
# such a fashion that the resource group is made online when appropriate.
#
# Input parameters
# Arg 1 - Group priority (an integer value from 0 thru 200)
# Arg 2 - Resource Group name
#
# Exit code:  0 - resource group made
#            !0 - exit code from mkrg command
#

smha_mkrg_enabled () {
    local Pri=$1
    local Name=$2
    local RC
    local cmd_out

    cmd_out=`mkrg -p $Pri -N Enabled $Name 2>&1`
    RC=$?

    if [ $RC -eq 0 ]; then
        smha_debug "Made resource group \"$Name\" Enabled"
    else
        smha_debug "Could not create resource group \"$Name\" Enabled, RC=$RC"
        smha_debug "mkrg output: $cmd_out"
    fi

    return $RC
}

# The following function is invoked by a Controlled script to create its resource group in
# such a fashion that the resource group remains offline until the smha --enable-rg is
# invoked for the named resource group
#
# Input parameters
# Arg 1 - Group priority (an integer value from 0 thru 200)
# Arg 2 - Resource Group name
#
# Exit code:  0 - resource group made
#            !0 - exit code from mkrg command
#

smha_mkrg_disabled () {
    local Pri=$1
    local Name=$2
    local RC
    local cmd_out

    cmd_out=`mkrg -p $Pri -N Disabled $Name 2>&1`
    RC=$?

    if [ $RC -eq 0 ]; then
        smha_debug "Made resource group \"$Name\" Disabled"
    else
        smha_debug "Could not create resource group \"$Name\" Disabled, RC=$RC"
        smha_debug "mkrg output: $cmd_out"
    fi

    return $RC
}

# The following function is used to get the IBM.Equivalency resource that is to be the target of
# an IBM.ServiceIP resource. This function attempts to return an equivalency of either IPv4 or IPv6
# addresses. If the desired type of equivalency is not available, the other type is returned.
# This function takes two arguments. It writes the equivalency name to stdout, so the function must
# be invoked like equ=$(smha_get_equ_target eth0 6)
#
# Input parameters
# Arg 1 - name of network interface to which the IBM.ServiceIP resource is to be assigned by TSA,
#         e.g. eth0, eth1
# Arg 2 - preferred type of equivalency, either 4 or 6.
#
# Output
# The equivalency name is written to stdout, or NULL on error.
#
# Exit code:  0 - equivalency name written to stdout
#            !0 - no equivalency found of either type
#
smha_get_equ_target () {
    local nic=$1
    local IPtype=$2
    local token
    local saveName=
    local prefName         # preferred equivalency name

    if [ "$IPtype" = "6" ]; then
        prefName="${nic}v6-equ"
    else
        prefName="${nic}-equ"
    fi

    # we find, at most, two resources

    for token in $(LC_ALL=C lsrsrc-api -s IBM.Equivalency::"Name LIKE '${nic}%-equ'"::Name 2> /dev/null); do
        if [ "$token" = "$prefName" ]; then
            echo $token
            return 0
        fi
        saveName=$token
    done

    if [ -n "$saveName" ]; then
        echo $saveName
        return 0
    fi

    return 1      # nothing found
}
